from py2neo import Graph, Node, Relationship
from influxdb import InfluxDBClient
from datetime import datetime, timedelta
from time import sleep


live=False

graph = Graph(password="admin")
graph.delete_all()  # clear out any entities from previous execution of the script

input("Create infrastructure topology graph. Click Enter to continue...\n")

# Create the infrastructure topology graph
computeNode1 = Node("ComputeNode", name="DC1-WaterShed", latitude="51.452014", longitude="-2.598169")
graph.create(computeNode1)
computeNode2 = Node("ComputeNode", name="DC2-EngineShed", latitude="51.449053", longitude="-2.583486")
graph.create(computeNode2)
computeNode3 = Node("ComputeNode", name="DC3-WhapingWharf", latitude="51.447594", longitude="-2.597066")
graph.create(computeNode3)

relationship = Relationship(computeNode1, "linkedTo", computeNode2)
graph.create(relationship)
relationship = Relationship(computeNode1, "linkedTo", computeNode3)
graph.create(relationship)
relationship = Relationship(computeNode2, "linkedTo", computeNode3)
graph.create(relationship)
# ----------------------------------------

input("Create media service topology graph. Click Enter to continue...\n")

# Create the service topology graph
nginxNode = Node("ServiceFunction", name="nginx")
graph.create(nginxNode)
minioNode = Node("ServiceFunction", name="minio")
graph.create(minioNode)
ueNode = Node("UserEquipment", name="ue")
graph.create(ueNode)

relationship = Relationship(nginxNode, "dependsOn", minioNode)
graph.create(relationship)
relationship = Relationship(ueNode, "uses", nginxNode)
graph.create(relationship)
# ----------------------------------------

input("Create media service instance topology graph. Click Enter to continue...\n")

# Create the service topology instance graph
nginxNodeInstance = Node("ServiceFunctionInstance", name="nginx_1")
graph.create(nginxNodeInstance)
minioNodeInstance = Node("ServiceFunctionInstance", name="minio_1")
graph.create(minioNodeInstance)

relationship = Relationship(nginxNodeInstance, "dependsOn", minioNodeInstance)
graph.create(relationship)

relationship = Relationship(nginxNodeInstance, "instanceOf", nginxNode)
graph.create(relationship)
relationship = Relationship(minioNodeInstance, "instanceOf", minioNode)
graph.create(relationship)
# ----------------------------------------

input("Create TOSCA topology graph. Click Enter to continue...\n")

# Create the endpoint topology graph
nginxEndpointNode1 = Node("Endpoint", name="nginx_1_ep1")
graph.create(nginxEndpointNode1)
nginxEndpointNode2 = Node("Endpoint", name="nginx_1_ep2")
graph.create(nginxEndpointNode2)
minioEndpointNode1 = Node("Endpoint", name="minio_1_ep1")
graph.create(minioEndpointNode1)
minioEndpointNode2 = Node("Endpoint", name="minio_1_ep2")
graph.create(minioEndpointNode2)

relationship = Relationship(nginxNodeInstance, "realisedBy", nginxEndpointNode1)
graph.create(relationship)
relationship = Relationship(nginxNodeInstance, "realisedBy", nginxEndpointNode2)
graph.create(relationship)
relationship = Relationship(minioNodeInstance, "realisedBy", minioEndpointNode1)
graph.create(relationship)
relationship = Relationship(minioNodeInstance, "realisedBy", minioEndpointNode2)
graph.create(relationship)

relationship = Relationship(nginxEndpointNode1, "hostedBy", computeNode1)
graph.create(relationship)
relationship = Relationship(nginxEndpointNode2, "hostedBy", computeNode2)
graph.create(relationship)
relationship = Relationship(minioEndpointNode1, "hostedBy", computeNode1)
graph.create(relationship)
relationship = Relationship(minioEndpointNode2, "hostedBy", computeNode3)
graph.create(relationship)

ueNode3 = Node("UserEquipmentEndpoint", name="UE3")
graph.create(ueNode3)
ueNode2 = Node("UserEquipmentEndpoint", name="UE2")
graph.create(ueNode2)

relationship = Relationship(ueNode3, "accessFrom", computeNode3)
graph.create(relationship)
relationship = Relationship(ueNode2, "accessFrom", computeNode2)
graph.create(relationship)
# ----------------------------------------

input("Click Enter to start populating graph with data properties from time-series monitoring\n")

db_client = InfluxDBClient(host='172.40.231.51', port=8086, database="MSDemo")  # defaults to localhost, port 8086
minio_measurements = tuple(measurement['name'] for measurement in db_client.get_list_measurements() if 'minio' in measurement['name'])  # get all measurements in regards to minio

if live:
    # use this line instead of the 2 lines above to work with real time data 
    print("Using live data\n")   
    current_datetime = datetime.now()  
else:
    # initial timestamp of nginx measurement in the snapshot
    print("Using snapshot data\n")       
    current_datetime = datetime.fromtimestamp(1528385420000000000 // 1000000000)  
    current_datetime = current_datetime + timedelta(seconds=10)

# find all endpoint nodes in the graph
endpoint_nodes = graph.nodes.match("Endpoint")

# Start populating graph based on data from influx
while True:
    #formatted_current_datetime = current_datetime.strftime("%Y-%m-%d %H:%M:%S")
    #formatted_old_datetime = (current_datetime - timedelta(seconds=10)).strftime("%Y-%m-%d %H:%M:%S")
    formatted_current_datetime = current_datetime
    formatted_old_datetime = current_datetime - timedelta(seconds=10)

    start_timestamp = int((current_datetime - timedelta(seconds=10)).timestamp()) * 10 ** 9  # in nanoseconds
    end_timestamp = int(current_datetime.timestamp()) * 10 ** 9  # in nanoseconds

    # go through all endpoints in the graph
    for endpoint_node in endpoint_nodes:
        endpoint_id = dict(endpoint_node)['name']

        # check if this is an nginx endpoint
        if 'nginx' in endpoint_id:
            print("Generating endpoint measurements for {0}".format(endpoint_id))
            result = db_client.query('SELECT mean(*) FROM "MSDemo"."autogen"."nginx" WHERE ipendpoint=\'{0}\' and time>={1} and time<{2}'.format(endpoint_id, start_timestamp, end_timestamp))

            print('SELECT mean(*) FROM "MSDemo"."autogen"."nginx" WHERE ipendpoint=\'{0}\' and time>={1} and time<{2}'.format(endpoint_id, start_timestamp, end_timestamp))
            try:
                actual_result = next(result.get_points())
                actual_result.pop('time')
                state_node = Node("State", name="State:"+endpoint_id, **actual_result)
                graph.create(state_node)
                graph.create(Relationship(endpoint_node, "monitoring", state_node, startFormatted=formatted_old_datetime, endFormatted=formatted_current_datetime))
                graph.create(Relationship(endpoint_node, "monitoring", state_node, start=start_timestamp, end=end_timestamp))                
                print("Added state node for endpoint {0}".format(endpoint_id))
            except StopIteration:
                print("No points retrieved to populate the graph for endpoint {0}".format(endpoint_id))
        # check if this is an minio endpoint
        elif 'minio' in endpoint_id:
            fields = {}  # all measurement fields from the minio tables are stored in here
            print("Generating endpoint measurements for {0}".format(endpoint_id))
            # take fields from all measurement tables related to minio (more than 1 found in the media service demo)
            for minio_measurement in minio_measurements:
                result = db_client.query('SELECT mean(*) FROM "MSDemo"."autogen"."{0}" WHERE ipendpoint=\'{1}\' and time>={2} and time<{3}'.format(minio_measurement, endpoint_id, start_timestamp, end_timestamp))
                try:
                    actual_result = next(result.get_points())
                    actual_result.pop('time')
                    for key in actual_result:
                        fields[minio_measurement + "_" + key] = actual_result[key]
                except StopIteration:
                    pass  # pass if no points found

            if len(fields) > 0:
                # create a state node with all fields found
                state_node = Node("State", name="State:" + endpoint_id, **fields)
                graph.create(state_node)
                graph.create(Relationship(endpoint_node, "monitoring", state_node, startFormatted=formatted_old_datetime, endFormatted=formatted_current_datetime))
                graph.create(Relationship(endpoint_node, "monitoring", state_node, start=start_timestamp, end=end_timestamp))      
                print("Added state node for endpoint {0}".format(endpoint_id))
            else:
                print("No points retrieved to populate the graph for endpoint {0}".format(endpoint_id))
    print('\n')

    sleep(10)
    
    if live:
        print("Using live data\n")          
        current_datetime = datetime.now()  # use this line instead of the line above to work with real time data
    else:
        print("Using snapshot data\n")  
        current_datetime = current_datetime + timedelta(seconds=10)

