# FLAME media service integration workshop

## Demonstration set-up notes

### Prerequisites

### Create the Vagrant host and log in

```shell
vagrant up
vagrant ssh
sudo su
```

### NATting (for Windows host)

* Chronograf: port 8888

* 2 NGINX installations (/…/services/nginx)
  * ports: 9180 and 9181

* NEO4J installations (/…/services/minio)
  * 7474 neo4j http browser
  * 7687 neo4j bolt 


* 2 MINIO installations (/…/services/minio)
  * ports: 9182 and 9183  

* Kapacitor API (for rule management): port 9092

### Create all containers, leave nginx1 and clmc started

```shell
# /vagrant/scripts/test/fixture.sh create -f /vagrant/src/test/clmctest/rspec.json
# lxc-ls
clmc-service host1        host2        host3        host4
```

```shell
/vagrant/scripts/test/fixture.sh stop -f /vagrant/src/test/clmctest/rspec.json -c host2
/vagrant/scripts/test/fixture.sh stop -f /vagrant/src/test/clmctest/rspec.json -c host3
/vagrant/scripts/test/fixture.sh stop -f /vagrant/src/test/clmctest/rspec.json -c host4
```

### Re-configure NGINX container and drop MSDemo DB

```shell
cd /vagrant/src/test/clmctest/workshopdemo/nginxMiniDemo
./initDemo.sh
```

## Mini NGINX demonstration

### Attach to container and install TELEGRAF (manually or otherwise)

`lxc-attach -n host1`

> Method for installing Telegraf is described in `/tmp/installTelegraf.sh`

### Send a simple metric via HTTP listener (whilst in NGINX container)

```shell
apt-get install curl
curl -i -XPOST 'http://172.40.231.51:8086/write?db=SimpleDemo' --data-binary 'MyServiceComponent requestCount=1'
```

This should return a 'HTTP/1.1 204 No Content' response

### View data point in CLMC

Go to `localhost:8888`...

Connect to "SimpleDemo" database.

### Create/Load dashboard in CLMC

Connect to "MSDemo" database instead.

Look at data explorer.

Create a graph on a dashboard and save dashboard:

```shell
wget http://172.40.231.51:8888/chronograf/v1/dashboards/1 -O - | jq '.'
```

Load one made earlier:

```shell
curl -i -XPOST -H "Content-Type: application/json" http://172.40.231.51:8888/chronograf/v1/dashboards -d @dashboards/NGINX_ep1_dash.json
```

## Main scenario demonstration

### Start the containers and check

```shell
/vagrant/scripts/test/fixture.sh start -f /vagrant/src/test/clmctest/rspec.json -c host2
/vagrant/scripts/test/fixture.sh start -f /vagrant/src/test/clmctest/rspec.json -c host3
/vagrant/scripts/test/fixture.sh start -f /vagrant/src/test/clmctest/rspec.json -c host4
lxc-ls
> clmc-service host1        host2        host3        host4
```

### Config chronograf dashboards

Go to Chronograf and config DB

```shell
curl -i -XPOST -H "Content-Type: application/json" http://172.40.231.51:8888/chronograf/v1/dashboards -d @dashboards/minio_dash.json
curl -i -XPOST -H "Content-Type: application/json" http://172.40.231.51:8888/chronograf/v1/dashboards -d @dashboards/nginx_dash.json
curl -i -XPOST -H "Content-Type: application/json" http://172.40.231.51:8888/chronograf/v1/dashboards -d @dashboards/sf_dash.json
curl -i -XPOST -H "Content-Type: application/json" http://172.40.231.51:8888/chronograf/v1/dashboards -d @dashboards/dc_dash.json
```

### Run demo initializer

Load in MPEG-DASH data and configure nginx to be proxy.

```shell
cd /vagrant/src/test/clmctest/workshopdemo
chmod +x initDemo.sh
./initDemo.sh
```

### Show the uploaded data in Minio web interface

MINIO 1
http://localhost:9182/minio/democontent/

MINIO 2
http://localhost:9183/minio/democontent/


### Set-up an alert (via Chronograf)

#### Set-up connection to Kapacitor (via dashboard; could use POST here during set-up)

#### Create a Slack app and configure a Webhook

We've done this already:
```
https://flamedemo-itinnov.slack.com/services/B99PACCLW
```

#### Configure a Slack alert end-point in Kapacitor

Enter your WebHook endpoint, for example:

```
channel : #clmc
webhook url: https://hooks.slack.com/services/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
```

#### Create a usage rule (manually, via Chronograf)

1. Click on the 'Alerting' icon (left panel on dashboard)
2. Click on 'Build Alert Rule'
3. Fill in the form info; a few highlights:
  - minio_network_sent_bytes_total, field: counter
  - Add a condition
  - Select alert handler (pre-configured slack)
  - Add a message, such as:

{{ .ID }} has exceeded network sent usage bytes ( {{ index .Fields "value" }} )

4. Save the rule

#### Optional: getting existing rules via Kapacitor HTTP API

```
http://localhost:9092/kapacitor/v1/tasks
```

#### POSTing a usage rule (via Kapacitor HTTP API)

This rule sets up an alert when any MINIO EP exceeds 1Mb of sent data

```
curl -i -XPOST 'http://172.40.231.51:9092/kapacitor/v1/tasks' -d @./rules/minio_bytes_sent.json
```

### Stream some video

Use the BitMovin' player: http://bitmovin.com/hls-mpeg-dash-test-player/

with these sources:

* http://localhost:9180/car-20120827-manifest.mpd
* http://localhost:9181/car-20120827-manifest.mpd

### Look at dashboards

http://localhost:8888/sources/1/dashboards

### Look at slack channel

https://flamedemo-itinnov.slack.com/messages/C99PABHRC/

### Tear-down

```
/vagrant/scripts/test/fixture.sh destroy -f /vagrant/src/test/clmctest/rspec.json
exit
exit
vagrant destroy
```